# SSEBop ESPA Open Source

## Overview
The operational Simplified Surface Energy Balance (SSEBop) Earth Resources Observation and Science center ([EROS](https://www.usgs.gov/earth-resources-observation-and-science-eros-center)) Science Processing Architecture ([ESPA](https://espa.cr.usgs.gov/)) Open Source project provides tools for processing satellite imagery data to generate evapotranspiration (ET) products. The tools include functionalities for:
- Processing ESPA files to generate ET fraction (ETf) and actual evapotranspiration (ETa) data.
- Filling gaps in ETf data using linear interpolation.
- Interpolating ESPA ETf data using reference ET data.
- Aggregating daily ETa data into monthly, annual, and seasonal ETa products.


The model is based on Python 3.x and open source modules. For raster manipulation the 'rasterio' and 'numpy' modules are used.
The code is organized in a python libray and includes following files:  
`gui.py` - front end graphical user interface  
`processing_steps.py` - backend aggregation and interpolation

More details are provided in the comments of each `*.py` file.

#### References:
Senay, G.B., Parrish, G.E.L., Schauer, M., Friedrichs, M., Khand, K., Boiko, O., Kagone, S., Dittmeier, R., Arab, S. and Ji, L., (2023). Improving the operational simplified surface energy balance evapotranspiration model using the forcing and normalizing operation. Remote Sensing, 15(1), 260.

## Citation
Kagone, S., Parrish, G.E.L., Senay, G.B., 2024, SSEBop ESPA Open Source. Geological Survey software release. https://doi.org/10.5066/P1EOTDQM

## Installation

### Step 1: Install Anaconda
Anaconda is a software bundle of Python and R for scientific computing and data science. It simplifies package management and deployment.

1. Download the Anaconda installer for your operating system from the [Anaconda website](https://www.anaconda.com/products/distribution).
2. Run the installer:
    - On Windows: Double-click the `.exe` file and follow the instructions.
    - On macOS: Open the `.pkg` file and follow the instructions.
    - On Linux: Open your terminal application, navigate to the directory containing the downloaded installer, and run:
      ```sh
      bash Anaconda3-<version>-Linux-x86_64.sh
      ```

### Step 2: Open Anaconda Prompt
1. After installing Anaconda, open the Anaconda Prompt:
    - On Windows: Search for "Anaconda Prompt" in the Start menu and open it.
    - On macOS and Linux: Open your terminal application.

### Step 3: Create a New Conda Environment
A conda environment is a separate environment for different projects, allowing you to manage dependencies easily.

1. Create a new conda environment named `ssebop_espa` with Python 3.8:
    ```sh
    conda create -n ssebop_espa python=3.8
    ```
2. Activate the new environment:
    ```sh
    conda activate ssebop_espa
    ```

### Step 4: Clone the GitLab Repository
Git is a version control system that lets you download (clone) the project repository from Gitlab to your local machine.

1. Navigate to the directory where you want to clone the repository by typing:
    ```sh
    cd path_to_your_directory
    ```
2. Clone the repository using Git:
    ```sh
    git clone https://code.usgs.gov/eros_hydro/ssebop_espa_opensource.git
    ```
3. Navigate into the project directory:
    ```sh
    cd ssebop_espa_open_source
    ```

### Step 5: Install the Module with pip
pip is a package installer for Python. You will use it to install the `ssebop_espa_open_source` module.

1. Ensure you are in the `ssebop_espa_open_source` directory.
2. Install the module using pip:
    ```sh
    pip install .
    ```

### Step 6: Run the Module
1. With the `ssebop_espa` environment activated, you can run the module using the following command:
    ```sh
    python -m ssebop_espa.gui
    ```

## Usage
The GUI application provides an interface to process ESPA files, fill gaps in ETf data, interpolate ESPA ETf data, and aggregate ETa data. Follow these steps to use the application:

### Process ESPA Files (1)
1. Navigate to the "Process ESPA Files" tab.
2. Click "Browse" to select the input directory containing the tar.gz files.
3. Click "Run Script" to start processing the files. The progress and status will be displayed.

### Gapfill ETf (2)
1. Navigate to the "Gapfill ETf" tab. 
2. Click "Browse" to select the input directory containing the ETf data. The input directory will be found in the parent folder i.e.” ..data/pathXXrowYY”.
3. Enter the number of days for gap filling. This is the amount of days before and after a given ESPA ETf overpass image that will be searched for good quality data and linearly interpolated to fill gaps.
4. Click "Run Script" to start the gap filling process. The progress and status will be displayed.

### Interpolate ESPA ETf (3)
1. Navigate to the "Interpolate ESPA ETf" tab.
2. Click "Browse" to select the input directory (the parent directory containing the gapfilled ETf data). The input directory will be found in the parent folder i.e.” ..data/pathXXrowYY”.
3. Click "Browse" to select the reference ET directory. Only daily climatology reference ET is supported at this time, e.g. a 20 year median daily reference ET in mm for the years 2004-2023 (20 year median).
4. Enter the reference ET file format. The only format accepted at this time is "pet_ddd". Greater flexibility in future versions of the gui for this option will feature in future versions.
5. Select whether the daily reference ET is a climatology. Only 'true' is accepted at this time per step #3.
6. Click "Run Script" to start the interpolation process. The progress and status will be displayed.

### Aggregate ET (4)
1. Navigate to the "Aggregate ET" tab.
2. Select the sub-tab for "Monthly ETa", "Annual ETa", or "Seasonal ETa".
3. For each sub-tab:
    - Click "Browse" to select the input directory containing the daily ETa data, i.e. ” ..data/pathXXrowYY/dailyETa”.
    - Enter the required parameters (e.g., year, start month, end month).
    - Click "Run Script" to start the aggregation process. The progress and status will be displayed.

## Authors and acknowledgment
**Gabriel Senay**1, **Stefanie Kagone**2, **Gabriel Parrish**3, **Matt Schauer**4.  
1 USGS EROS Center, North Central Climate Science Center, USA  
2 ASRC Federal Data Solutions,EROS Center   
3 Innovate!, Inc., EROS Center  
4 Formerly KBR, Contractor to USGS EROS Center

## License
CC0 1.0 Universal

## Project Status
ssebop_espa_opensource 0.0.2 is complete and published under tag v0.0.2

ssebop_espa_opensource 0.0.3 is under development.

## Additional Information
- Ensure that you have the necessary dependencies installed in your conda environment.
- For more detailed information on each function and its parameters, refer to the source code and inline comments/docstrings.

If you encounter any issues or have questions, please refer to the [ssebop_espa_opensource repository](https://code.usgs.gov/eros_hydro/ssebop_espa_opensource) for additional documentation and support.
