# Load dependencies

import platform
# print(platform.python_version())
import requests
import os
import ast
import getpass
import time
import datetime
import json
from typing import Union
from pprint import pprint
import argparse

# Enter ERS credentials:
parser = argparse.ArgumentParser()
parser.add_argument('-e', '--environment', required=False, default='ops', help='ESPA Environemnt')
parser.add_argument('-u', '--username', required=False, default=False, help='ERS username')
parser.add_argument('-id', '--orderid', nargs="*", required=False, default=False, help='ESPA order ID')
parser.add_argument('-o', '--outdir', required=False, default=False, help='Output directory')
args = parser.parse_args()
env = args.environment
username = args.username
outdir = args.outdir

orderids = args.orderid  # Example: espa-sarab@contractor.usgs.gov-06032019-154839-315

print('Enter ERS password:')
password = getpass.getpass()

# ## Host service and credentials

espa_env = {
    "dev": "https://espa-dev.cr.usgs.gov",
    "tst": "https://espa-tst.cr.usgs.gov",
    "ops": "https://espa.cr.usgs.gov"
}

host = espa_env[env] + "/api/v1/"


# ### ESPA API


def espa_api(endpoint, verb='get', body=None, uauth=None):
    """ Suggested simple way to interact with the ESPA JSON REST API """
    auth_tup = uauth if uauth else (username, password)
    response = getattr(requests, verb)(host + endpoint, auth=auth_tup, json=body)
    print('{} {}'.format(response.status_code, response.reason))
    data = response.json()
    if isinstance(data, dict):
        messages = data.pop("messages", None)
        if messages:
            print(json.dumps(messages, indent=4))
    try:
        response.raise_for_status()
    except Exception as e:
        print(e)
        return None
    else:
        return data


# ## Download Function


def retrieve_order(outdir: str, order_url: str):
    """
    Download the order
    :param outdir: Full path to the order download folder
    :param order_url: Order download URL
    :return:
    """

    if not os.path.exists(outdir):
        os.makedirs(outdir)

    print("Initiating download...")

    t = time.time()

    for order in order_url:
        order_id = order.split("/")[-1]
        outfile = outdir + os.sep + order_id

        resp = requests.get(order, stream=True)

        print(resp.status_code)

        if resp.status_code == "ordered" or resp.status_code == 200:
            print("Starting download...")

            with open(outfile, "wb") as f:
                for chunk in resp.iter_content(chunk_size=2048):
                    f.write(chunk)

                if os.path.exists(outfile):
                    print(os.path.getsize(outfile) / 1024 / (time.time() - t) / 1024)

        else:
            print("Could not download {}".format(order))

            print("Error {}".format(str(resp.status_code)))

        print("Download of {} complete.".format(order_id))

    return None


# ### Check Order Status


for orderid in orderids:
    resp = espa_api('order-status/{}'.format(orderid))
    print(json.dumps(resp, indent=4))

# ### Download orders with "status": "complete"

for orderid in orderids:
    order_url = list()
    print("downloading %s" % orderid)
    resp = espa_api('item-status/{0}'.format(orderid), body={'status': 'complete'})

    for item in resp[orderid]:
        order_url.append(item.get('product_dload_url'))
    retrieve_order(outdir, order_url)

print("Downloaded files can be found at {}".format(outdir))
